"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const internals_1 = require("../internals");
/**
 * Creates an empty tree. If no comparator function is supplied, keys are compared using logical less-than and
 * greater-than operations, which will generally only be suitable for numeric or string keys.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {Comparator<K>} [comparator] A comparison function, taking two keys, and returning a value less than 0 if the
 *                                     first key is smaller than the second, a value greater than 0 if the first key is
 *                                     greater than the second, or 0 if they're the same.
 * @returns {RedBlackTree<K, V>} An empty tree
 */
function empty(comparator) {
    return internals_1.createTree(false, comparator);
}
exports.empty = empty;
/**
 * Determines whether the input argument is an instance of a Collectable.js RedBlackTree structure.
 *
 * @export
 * @param {RedBlackTree<K, V>} arg The input value to check
 * @returns {boolean} True if the input value is a RedBlackTree, otherwise false
 */
function isRedBlackTree(arg) {
    return typeof arg === 'object' && arg !== null && arg instanceof internals_1.RedBlackTreeImpl;
}
exports.isRedBlackTree = isRedBlackTree;

//# sourceMappingURL=empty.js.map
