"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("./node");
const path_1 = require("./path");
class RedBlackTreeIterator {
    constructor(_current, _reversed = false) {
        this._current = _current;
        this._reversed = _reversed;
        this._begun = false;
        if (_current.isActive()) {
            _current.next = _reversed ? 2 /* RIGHT */ : 1 /* LEFT */; // indicates the already-traversed branch
        }
    }
    _next(reverse) {
        var current = this._current;
        var result = { done: false, value: node_1.NONE };
        if (current.isNone()) {
            result.done = true;
            return result;
        }
        if (!this._begun) {
            this._begun = true;
            result.value = current.node;
            return result;
        }
        var done = false, node, canEmit = false;
        do {
            switch (current.next) {
                case 0 /* NONE */:
                    if (reverse) {
                        node = current.node._right;
                        current.next = 2 /* RIGHT */;
                    }
                    else {
                        node = current.node._left;
                        current.next = 1 /* LEFT */;
                    }
                    if (!node_1.isNone(node)) {
                        current = path_1.PathNode.next(node, current, 0 /* NONE */);
                    }
                    break;
                case 1 /* LEFT */:
                    if (reverse) {
                        current = current.release();
                    }
                    else if (canEmit) {
                        result.value = current.node;
                        done = true;
                    }
                    else {
                        node = current.node._right;
                        if (node_1.isNone(node)) {
                            current = current.release();
                        }
                        else {
                            current.next = 2 /* RIGHT */;
                            current = path_1.PathNode.next(node, current, 0 /* NONE */);
                        }
                    }
                    break;
                case 2 /* RIGHT */:
                    if (!reverse) {
                        current = current.release();
                    }
                    else if (canEmit) {
                        result.value = current.node;
                        done = true;
                    }
                    else {
                        node = current.node._left;
                        if (node_1.isNone(node)) {
                            current = current.release();
                        }
                        else {
                            current.next = 1 /* LEFT */;
                            current = path_1.PathNode.next(node, current, 0 /* NONE */);
                        }
                    }
                    break;
            }
            if (current.isNone()) {
                result.done = true;
                result.value = node_1.NONE;
                done = true;
            }
            else {
                canEmit = true;
            }
        } while (!done);
        this._current = current;
        return result;
    }
    next() {
        return this._next(this._reversed);
    }
    previous() {
        return this._next(!this._reversed);
    }
    [Symbol.iterator]() {
        return this;
    }
}
exports.RedBlackTreeIterator = RedBlackTreeIterator;

//# sourceMappingURL=iterator.js.map
